AND = 'AND'
FROM = 'FROM'
INSERT = 'INSERT'
LIKE = 'LIKE'
LIMIT = 'LIMIT'
OFFSET = 'OFFSET'
OR = 'OR'
SELECT = 'SELECT'
SET = 'SET'
UPDATE = 'UPDATE'
WHERE = 'WHERE'


DATABASES = ['postgres', 'sqlite']

POSTGRES = 'postgres'
POSTGRES_LIVE_DSN = 'postgres://microstudian:eldorad0@localhost:5432/microstudio'

SQLITE = 'sqlite'
SQLITE_LIVE_DSN = 'sqlite://amebo.db'